@@ -31,9 +31,11 @@ import (
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/tagged/taggedimpl"
 
 	// Developer preview features
+	_ "github.com/v2fly/v2ray-core/v5/app/commander/webcommander"
 	_ "github.com/v2fly/v2ray-core/v5/app/instman"
 	_ "github.com/v2fly/v2ray-core/v5/app/observatory"
-	_ "github.com/v2fly/v2ray-core/v5/app/restfulapi"
+	_ "github.com/v2fly/v2ray-core/v5/app/persistentstorage/filesystemstorage"
+	_ "github.com/v2fly/v2ray-core/v5/app/tun"
 
 	// Inbound and outbound proxies.
 	_ "github.com/v2fly/v2ray-core/v5/proxy/blackhole"
@@ -53,6 +55,9 @@ import (
 	_ "github.com/v2fly/v2ray-core/v5/proxy/vlite/inbound"
 	_ "github.com/v2fly/v2ray-core/v5/proxy/vlite/outbound"
 
+	_ "github.com/v2fly/v2ray-core/v5/proxy/hysteria2"
+	_ "github.com/v2fly/v2ray-core/v5/proxy/shadowsocks2022"
+
 	// Transports
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/domainsocket"
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/grpc"
@@ -71,7 +76,18 @@ import (
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/request/assembler/simple"
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/request/roundtripper/httprt"
 
+	_ "github.com/v2fly/v2ray-core/v5/transport/internet/request/assembler/packetconn"
+
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/request/stereotype/meek"
+	_ "github.com/v2fly/v2ray-core/v5/transport/internet/request/stereotype/mekya"
+
+	_ "github.com/v2fly/v2ray-core/v5/transport/internet/dtls"
+
+	_ "github.com/v2fly/v2ray-core/v5/transport/internet/httpupgrade"
+
+	_ "github.com/v2fly/v2ray-core/v5/transport/internet/hysteria2"
+
+	_ "github.com/v2fly/v2ray-core/v5/transport/internet/tlsmirror/server"
 
 	// Transport headers
 	_ "github.com/v2fly/v2ray-core/v5/transport/internet/headers/http"
@@ -107,4 +123,24 @@ import (
 	_ "github.com/v2fly/v2ray-core/v5/proxy/shadowsocks/simplified"
 	_ "github.com/v2fly/v2ray-core/v5/proxy/socks/simplified"
 	_ "github.com/v2fly/v2ray-core/v5/proxy/trojan/simplified"
+
+	// Subscription Supports
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/subscriptionmanager"
+
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/subscriptionmanager/command"
+
+	// Subscription Containers: general purpose
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/containers/base64urlline"
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/containers/dataurlsingle"
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/containers/jsonfieldarray"
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/containers/jsonfieldarray/jsonified"
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/containers/urlline"
+
+	// Subscription Fetchers
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/documentfetcher/dataurlfetcher"
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/documentfetcher/httpfetcher"
+
+	// Subscription Entries Converters
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/entries/nonnative"
+	_ "github.com/v2fly/v2ray-core/v5/app/subscription/entries/outbound" // Natively Supported Outbound Format
 )
