@@ -17,6 +17,8 @@ import (
 
 	core "github.com/v2fly/v2ray-core/v5"
 	"github.com/v2fly/v2ray-core/v5/common"
+	"github.com/v2fly/v2ray-core/v5/common/environment"
+	"github.com/v2fly/v2ray-core/v5/common/environment/envctx"
 	"github.com/v2fly/v2ray-core/v5/common/net"
 	"github.com/v2fly/v2ray-core/v5/common/session"
 	"github.com/v2fly/v2ray-core/v5/transport/internet"
@@ -38,12 +40,25 @@ func init() {
 	common.Must(internet.RegisterTransportDialer(protocolName, Dial))
 }
 
-type dialerCanceller func()
+type transportConnectionState struct {
+	scopedDialerMap    map[net.Destination]*grpc.ClientConn
+	scopedDialerAccess sync.Mutex
+}
 
-var (
-	globalDialerMap    map[net.Destination]*grpc.ClientConn
-	globalDialerAccess sync.Mutex
-)
+func (t *transportConnectionState) IsTransientStorageLifecycleReceiver() {
+}
+
+func (t *transportConnectionState) Close() error {
+	t.scopedDialerAccess.Lock()
+	defer t.scopedDialerAccess.Unlock()
+	for _, conn := range t.scopedDialerMap {
+		_ = conn.Close()
+	}
+	t.scopedDialerMap = nil
+	return nil
+}
+
+type dialerCanceller func()
 
 func dialgRPC(ctx context.Context, dest net.Destination, streamSettings *internet.MemoryStreamConfig) (net.Conn, error) {
 	grpcSettings := streamSettings.ProtocolSettings.(*Config)
@@ -70,25 +85,36 @@ func dialgRPC(ctx context.Context, dest net.Destination, streamSettings *interne
 }
 
 func getGrpcClient(ctx context.Context, dest net.Destination, dialOption grpc.DialOption, streamSettings *internet.MemoryStreamConfig) (*grpc.ClientConn, dialerCanceller, error) {
-	globalDialerAccess.Lock()
-	defer globalDialerAccess.Unlock()
+	transportEnvironment := envctx.EnvironmentFromContext(ctx).(environment.TransportEnvironment)
+	state, err := transportEnvironment.TransientStorage().Get(ctx, "grpc-transport-connection-state")
+	if err != nil {
+		state = &transportConnectionState{}
+		transportEnvironment.TransientStorage().Put(ctx, "grpc-transport-connection-state", state)
+		state, err = transportEnvironment.TransientStorage().Get(ctx, "grpc-transport-connection-state")
+		if err != nil {
+			return nil, nil, newError("failed to get grpc transport connection state").Base(err)
+		}
+	}
+	stateTyped := state.(*transportConnectionState)
 
-	if globalDialerMap == nil {
-		globalDialerMap = make(map[net.Destination]*grpc.ClientConn)
+	stateTyped.scopedDialerAccess.Lock()
+	defer stateTyped.scopedDialerAccess.Unlock()
+
+	if stateTyped.scopedDialerMap == nil {
+		stateTyped.scopedDialerMap = make(map[net.Destination]*grpc.ClientConn)
 	}
 
 	canceller := func() {
-		globalDialerAccess.Lock()
-		defer globalDialerAccess.Unlock()
-		delete(globalDialerMap, dest)
+		stateTyped.scopedDialerAccess.Lock()
+		defer stateTyped.scopedDialerAccess.Unlock()
+		delete(stateTyped.scopedDialerMap, dest)
 	}
 
-	// TODO Should support chain proxy to the same destination
-	if client, found := globalDialerMap[dest]; found && client.GetState() != connectivity.Shutdown {
+	if client, found := stateTyped.scopedDialerMap[dest]; found && client.GetState() != connectivity.Shutdown {
 		return client, canceller, nil
 	}
 
-	conn, err := grpc.Dial(
+	conn, err := grpc.NewClient(
 		dest.Address.String()+":"+dest.Port.String(),
 		dialOption,
 		grpc.WithConnectParams(grpc.ConnectParams{
@@ -117,6 +143,14 @@ func getGrpcClient(ctx context.Context, dest net.Destination, dialOption grpc.Di
 			return internet.DialSystem(detachedContext, net.TCPDestination(address, port), streamSettings.SocketSettings)
 		}),
 	)
-	globalDialerMap[dest] = conn
+	canceller = func() {
+		stateTyped.scopedDialerAccess.Lock()
+		defer stateTyped.scopedDialerAccess.Unlock()
+		delete(stateTyped.scopedDialerMap, dest)
+		if err != nil {
+			conn.Close()
+		}
+	}
+	stateTyped.scopedDialerMap[dest] = conn
 	return conn, canceller, err
 }
