@@ -2,6 +2,7 @@ package request
 
 import (
 	"context"
+	"io"
 
 	"github.com/v2fly/v2ray-core/v5/common"
 )
@@ -36,3 +37,12 @@ type Request struct {
 type Response struct {
 	Data []byte
 }
+
+type OptionSupportsStreamingResponse interface {
+	RoundTripperOption
+	GetResponseWriter() io.Writer
+}
+
+type OptionSupportsStreamingResponseExtensionFlusher interface {
+	Flush()
+}
