@@ -294,6 +294,13 @@ func (c *Config) GetTLSConfig(opts ...Option) *tls.Config {
 		}
 	}
 
+	if len(c.Ciphersuites) > 0 {
+		config.CipherSuites = make([]uint16, 0, len(c.Ciphersuites))
+		for _, cs := range c.Ciphersuites {
+			config.CipherSuites = append(config.CipherSuites, uint16(cs))
+		}
+	}
+
 	return config
 }
 
