@@ -1,5 +1,5 @@
-//go:build windows && !confonly
-// +build windows,!confonly
+//go:build windows
+// +build windows
 
 package tls
 
@@ -7,7 +7,7 @@ import "crypto/x509"
 
 func (c *Config) getCertPool() (*x509.CertPool, error) {
 	if c.DisableSystemRoot {
-		return c.loadSelfCertPool()
+		return c.loadSelfCertPool(Certificate_AUTHORITY_VERIFY)
 	}
 
 	return nil, nil
